/*
 *	MyGLCanvas.cpp
 *	OpenGL Animation Demo
 *
 *	Ross Ashman 4 Jan 2005.
 *	Copyright (c) 2005 Ross Ashman. All rights reserved.
 *  
 */

// MyGLCanvas.cpp: implementation of the MyGLCanvas class.
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Headers
//////////////////////////////////////////////////////////////////////
#include "wx/wxprec.h"

#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include <gl/glu.h>
#include <math.h>

#include "MyGLCanvas.h"
#include <wx/list.h>

//////////////////////////////////////////////////////////////////////
// Constants
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Event Tables and Other Macros for wxWindows
//////////////////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE(MyGLCanvas, wxGLCanvas)
	EVT_SIZE(MyGLCanvas::OnSize)
	EVT_PAINT(MyGLCanvas::OnPaint)
	EVT_LEFT_DOWN(MyGLCanvas::OnLeftMouse)
    EVT_LEFT_UP(MyGLCanvas::OnLeftMouseUp)	
	EVT_RIGHT_DOWN(MyGLCanvas::OnRightMouse)
    EVT_RIGHT_UP(MyGLCanvas::OnRightMouseUp)	
	EVT_MOTION(MyGLCanvas::OnMouseMove)	
	EVT_ERASE_BACKGROUND(MyGLCanvas::OnEraseBackground)
	EVT_IDLE(MyGLCanvas::OnIdle)
END_EVENT_TABLE()

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MyGLCanvas::MyGLCanvas(wxWindow* parent,wxPanel *_panel, 
                                wxWindowID id, 
                                const wxPoint& pos, 
                                const wxSize& size, 
                                long style, 
                                const wxString& name,
                                int* attribList, 
                                const wxPalette& palette)
: wxGLCanvas(parent, id, pos, size, style, name)
{
    panel = _panel;
    testLabel = NULL;
    WxCheckBoxRotation = NULL;
	InitGL();
	drag_point = false;
	drag_point_index = -1;

	// Set GL Viewport
	int w, h;
	GetClientSize(&w, &h);

	if(GetContext())
	{
		SetCurrent();
		glViewport(0, 0, (GLint)w, (GLint)h);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		//gluPerspective(45.0, (float)w/(float)h, 1.0, 1000.0);
		gluPerspective(0.0, (float)w/(float)h, 0.0, 0.0);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}
	
    wxWindowList &lista = panel->GetChildren();
	wxWindowListNode *node = NULL;
	node = lista.GetFirst();
	while(node->GetData()->GetName()!="testLabel") node = node->GetNext();
	
	if(node->GetData()->GetName()=="testLabel") testLabel = (wxStaticText*) node->GetData();
	
	node = lista.GetFirst();
	while(node->GetData()->GetName()!="WxCheckBoxRotation") node = node->GetNext();
	
    if(node->GetData()->GetName()=="WxCheckBoxRotation") WxCheckBoxRotation = (wxCheckBox*) node->GetData();
    
    /*node = lista.GetFirst();
	while(node->GetData()->GetName()!="WxCheckBoxZoom") node = node->GetNext();

    if(node->GetData()->GetName()=="WxCheckBoxZoom") WxCheckBoxZoom = (wxCheckBox*) node->GetData();*/
    
}

MyGLCanvas::~MyGLCanvas()
{

}

//////////////////////////////////////////////////////////////////////
// Method Implementations
//////////////////////////////////////////////////////////////////////
void MyGLCanvas::InitGL()
{
	
	glShadeModel(GL_SMOOTH);                 // Enables Smooth Color Shading
    glClearDepth(1.0);                       // Depth Buffer Setup
    glEnable(GL_DEPTH_TEST);                 // Enable Depth Buffer
    glDepthFunc(GL_LESS);		           // The Type Of Depth Test To Do
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);   //Realy Nice perspective calculations
    
    RotX=0;
    RotY=0;
    Depth = 0;
    
    
}

void MyGLCanvas::Draw()
{
    //this->Refresh();  //Refresh() seems to chew up more resources??! Just do a Paint instead!
    wxClientDC dc(this);
    Paint(dc);
}

void MyGLCanvas::Paint(wxDC &dc)
{
	if(!GetContext())
		return;

	SetCurrent();
    
    glClearColor(BColor.R, BColor.G, BColor.B, 1.0); 	   // White Background
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    
    //glTranslated(0.0, 0.0, Depth);
    glRotatef(RotX, 1.0, 0.0, 0.0);
    glRotatef(RotY, 0.0, 1.0, 0.0);

    glPointSize(10);
    glEnable(GL_POINT_SMOOTH);
    glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

    /*
    glBegin(GL_POINTS);
        glColor3f(1,0,0);
        glVertex3f(0.0,0.0,0.0);
        glColor3f(0,1,0);
        glVertex3f(1.0,0.0,0.0);
        glColor3f(0,0,1);
        glVertex3f(0.0,1.0,0.0);
        glColor3f(0,1,1);
        glVertex3f(0.0,0.0,-1.0);
    glEnd();
    */


    glBegin(GL_POINTS);
        glColor3f(1,1,1);
        
        for(int i(0);i<bezier.br_unesenih_tocaka;++i){
           glVertex3f(bezier.controlPoints[i][0],bezier.controlPoints[i][1],bezier.controlPoints[i][2]);
        }
    glEnd();
    

    
    if(bezier.br_tocaka!=0 && bezier.br_unesenih_tocaka==bezier.br_tocaka){
          glLineWidth(3);
          glEnable(GL_LINE_SMOOTH);
          glHint(GL_LINE_SMOOTH_HINT,GL_NICEST);
          glEnable(GL_BLEND);
          glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
          
        glBegin(GL_LINE_STRIP);
          glColor3f(0,1,0);
          
          for(int i(0);i<bezier.br_unesenih_tocaka;++i){
             glVertex3f(bezier.controlPoints[i][0],bezier.controlPoints[i][1],bezier.controlPoints[i][2]);
          }
        glEnd();
         
        glColor3f(1,0,0);
    
        glMap1f(GL_MAP1_VERTEX_3,0.0,1.0,3,bezier.br_tocaka,&(bezier.controlPoints[0][0]));
        glEnable(GL_MAP1_VERTEX_3);

        glMapGrid1f(100,0.0,1.0);
        glEvalMesh1(GL_LINE,0,100);
        
    }


    
    
    // Flush the GL Pipeline
	//glFlush();

	// Swap Your Buffers
	SwapBuffers();	
    
}


//////////////////////////////////////////////////////////////////////
// Event Handler Implementations
//////////////////////////////////////////////////////////////////////

void MyGLCanvas::OnPaint(wxPaintEvent& event)
{
	wxPaintDC dc(this);
	Paint(dc);
	event.Skip();
}

void MyGLCanvas::OnSize(wxSizeEvent& event)
{
	// This is necessary to update the context on some platforms
	wxGLCanvas::OnSize(event);

	// Set the GL Viewport (not called by wxGLCanvas::OnSize on all Platforms)
	int w, h;
	GetClientSize(&w, &h);
	if(GetContext())
	{
		SetCurrent();
		glViewport(0, 0, (GLint)w, (GLint)h);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(0.0, (float)w/(float)h, 0.0, 0.0);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}
}

void MyGLCanvas::OnLeftMouse(wxMouseEvent& event)
{
    	int w, h;
	    GetClientSize(&w, &h);        
        MouseButton = 1;
        event.GetPosition(&Xcoord, &Ycoord);
        if(bezier.br_unesenih_tocaka < bezier.br_tocaka){
            bezier.controlPoints[bezier.br_unesenih_tocaka][0] = (Xcoord-w/2.0)/(w/2.0);
            bezier.controlPoints[bezier.br_unesenih_tocaka][1] = (h/2.0-Ycoord)/(h/2.0);
            bezier.controlPoints[bezier.br_unesenih_tocaka][2] = 0.0;
            ++bezier.br_unesenih_tocaka;
            Draw();
        }else{
               //treba provjeriti je li korisnik mozda kliknuo na neku tocku kontrolnog poligona
               for(int i(0);i<bezier.br_tocaka;++i){ 
                  if((Xcoord-w/2.0)/(w/2.0) > (bezier.controlPoints[i][0]- 0.03) &&
                     (Xcoord-w/2.0)/(w/2.0) < (bezier.controlPoints[i][0]+ 0.03) &&
                     (h/2.0-Ycoord)/(h/2.0) > (bezier.controlPoints[i][1]- 0.03) &&
                     (h/2.0-Ycoord)/(h/2.0) < (bezier.controlPoints[i][1]+ 0.03)
                      ){
                            //kliknuli smo vrlo blizu neke tocke
                            drag_point = true;
                            drag_point_index = i;
	                        break;
                        }
               }
            }
        
}

void MyGLCanvas::OnRightMouse(wxMouseEvent& event)
{        
        MouseButton = 2;
        event.GetPosition(&Xcoord, &Ycoord);
        //event.GetPosition(&Dummy, &Zcoord);        
}

void MyGLCanvas::OnLeftMouseUp(wxMouseEvent& event)
{        
        MouseButton = 0;
        drag_point = false;
        drag_point_index = -1;
}

void MyGLCanvas::OnRightMouseUp(wxMouseEvent& event)
{        
        MouseButton = 0;
}

void MyGLCanvas::OnMouseMove(wxMouseEvent& event)
{
    int w, h;
	GetClientSize(&w, &h);
	
    if (event.Dragging() == true)
    {
        event.GetPosition(&X, &Y);

        if (MouseButton == 2 && WxCheckBoxRotation->IsChecked()==true)
        {
                RotX=RotX + (GLfloat)(Y - Ycoord)/2;
                RotY=RotY + (GLfloat)(X - Xcoord)/2;
                Xcoord = X;
                Ycoord = Y;
        }
        /*
        if (MouseButton == 2 && WxCheckBoxZoom->IsChecked()==true)
        {
                Depth = Depth - (GLfloat)(Y-Zcoord)/5;
                Zcoord = Y;
        }
        */
        
        if(MouseButton == 1 && drag_point == true){
            bezier.controlPoints[drag_point_index][0] = (X-w/2.0)/(w/2.0);
            bezier.controlPoints[drag_point_index][1] = (h/2.0-Y)/(h/2.0);
        }
        Draw(); // Causes an event to be generated to repaint the window.

    }
    event.GetPosition(&X, &Y);
    wxString s;
    s.Printf("X = %f : Y = %f",(X-w/2.0)/(w/2.0),(h/2.0-Y)/(h/2.0));
	if(testLabel)testLabel->SetLabel(s);
	
        
}

void MyGLCanvas::OnEraseBackground(wxEraseEvent& event)
{
	// Do Nothing to Avoid Flashing on MSW
}

void MyGLCanvas::OnIdle(wxIdleEvent& event)
{
	//this->Refresh();  // invoking this chews up lots of cpu cycles, not needed for static pictures, onpaint is sufficient
}


void MyGLCanvas::Clear(){
    for(int i(0);i<bezier.br_tocaka;++i){
        bezier.controlPoints[i][0] = 0.0;
        bezier.controlPoints[i][1] = 0.0;
        bezier.controlPoints[i][2] = 0.0;
    }
    bezier.br_tocaka = bezier.br_unesenih_tocaka = 0;
    RotX = RotY = 0.0;
    
    Draw();
}

